package com.zhiyun.project.item.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.zhiyun.project.item.domain.entity.Device;
import com.zhiyun.project.item.domain.vo.CoordinateVo;
import com.zhiyun.project.item.domain.vo.DeviceVo;
import com.zhiyun.project.item.domain.vo.OptionsVo;
import com.zhiyun.project.item.mapper.DeviceMapper;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class DeviceService extends ServiceImpl<DeviceMapper, Device> {

    public DeviceVo getDeviceData(Integer id){
        Device device = baseMapper.selectById(id);
        if(device == null)
            return null;

        DeviceVo deviceVo = new DeviceVo();
        BeanUtils.copyProperties(device, deviceVo);

        LocalDateTime now = LocalDateTime.now();
        if(Duration.between(device.getLatestTime(), now).toMinutes() < 1){
            deviceVo.setStatue(1);
        } else {
            deviceVo.setStatue(0);
        }

        return deviceVo;
    }

    public List<CoordinateVo> getCoordinateData(){
        // 查询所有设备数据
        List<Device> deviceList = baseMapper.selectList(null);

        // 如果设备列表为空，直接返回空列表
        if (deviceList == null || deviceList.isEmpty()) {
            return null;  // 返回空的列表
        }

        // 使用流式 API 转换 Device 列表为 CoordinateVo 列表
        return deviceList.stream().map(device -> {
            CoordinateVo coordinateVo = new CoordinateVo();
            coordinateVo.setDeviceid(device.getDeviceid());
            coordinateVo.setCoordinate(new Double[]{device.getX(), device.getY()});

            LocalDateTime now = LocalDateTime.now();
            if (Duration.between(device.getLatestTime(), now).toMinutes() < 1) {
                coordinateVo.setStatue(1);
            } else {
                coordinateVo.setStatue(0);
            }

            return coordinateVo;
        }).collect(Collectors.toList());
    }

    public int insetDeviceData(JSONObject jsonObject) {
        if (jsonObject == null || jsonObject.getString("deviceid") == null) {
            return -1;
        }
        // 创建 Device 实体对象
        Device device = new Device();

        device.setDeviceid(jsonObject.getString("deviceid"));

        if(jsonObject.getDouble("lat")!=null)
            device.setLat(jsonObject.getDouble("lat"));

        if(jsonObject.getDouble("lon")!=null)
            device.setLon(jsonObject.getDouble("lon"));

        if(jsonObject.getDouble("x")!=null)
            device.setX(jsonObject.getDouble("x"));

        if(jsonObject.getDouble("y")!=null)
            device.setX(jsonObject.getDouble("y"));

        if(jsonObject.getDouble("z")!=null)
            device.setX(jsonObject.getDouble("z"));

        if(jsonObject.getInteger("f")!=null)
            device.setX(jsonObject.getInteger("f"));


        // 获取原始经纬度
        if(device.getLat() != null && device.getLat() != null){

            Double lat = device.getLat();
            Double lon = device.getLon();

            // 调用坐标转换方法
            String[] convertedCoords = convertCoordinates(lat, lon);
            if (convertedCoords != null && convertedCoords.length == 2) {
                // 更新 Device 对象中的 x, y 坐标
                device.setLon(Double.parseDouble(convertedCoords[0]));  // 转换后的经度
                device.setLat(Double.parseDouble(convertedCoords[1]));  // 转换后的纬度
            } else {
                System.out.println("坐标转换失败，无法更新设备坐标");
                return -1; // 如果转换失败，返回失败标志
            }
        }


        // 设置当前时间
        device.setLatestTime(LocalDateTime.now());

        // 判断是否已有该设备记录，若无则插入新记录
        Device judgeObject = baseMapper.getDeviceById(device.getDeviceid());
        if (judgeObject == null) {
            return baseMapper.insert(device);
        } else {
            device.setId(judgeObject.getId());
            return baseMapper.updateById(device);
        }
    }

    public List<OptionsVo> getDeviceOptions() {
        return baseMapper.getDeviceOptions();
    }

    // 坐标转换：WGS-84 到 GCJ-02
    public String[] convertCoordinates(double lat, double lon) {
        // 判断是否在中国境外
        if (outOfChina(lat, lon)) {
            return new String[]{String.valueOf(lon), String.valueOf(lat)}; // 如果在中国境外，直接返回原坐标
        }

        // 计算坐标偏移量
        double dLat = transformLat(lon - 105.0, lat - 35.0);
        double dLon = transformLon(lon - 105.0, lat - 35.0);

        // 根据偏移量调整经纬度
        double radLat = lat / 180.0 * Math.PI;
        double magic = Math.sin(radLat);
        magic = 1 - 0.00669342162296594323 * magic * magic;
        double sqrtMagic = Math.sqrt(magic);

        dLat = (dLat * 180.0) / (6378245.0 * (1 - 0.00669342162296594323) / (magic * sqrtMagic) * Math.PI);
        dLon = (dLon * 180.0) / (6378245.0 / sqrtMagic * Math.cos(radLat) * Math.PI);

        double mgLat = lat + dLat;
        double mgLon = lon + dLon;

        return new String[]{String.valueOf(mgLon), String.valueOf(mgLat)};
    }

    // 判断是否在中国境外
    private boolean outOfChina(double lat, double lon) {
        return lon < 72.004 || lon > 137.8347 || lat < 0.8293 || lat > 55.8271;
    }

    // 坐标转换公式
    private double transformLat(double x, double y) {
        double ret = -100.0 + 2.0 * x + 3.0 * y + 0.2 * y * y + 0.1 * x * y + 0.2 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * Math.PI) + 20.0 * Math.sin(2.0 * x * Math.PI)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(y * Math.PI) + 40.0 * Math.sin(y / 3.0 * Math.PI)) * 2.0 / 3.0;
        ret += (160.0 * Math.sin(y / 12.0 * Math.PI) + 320.0 * Math.sin(y * Math.PI / 30.0)) * 2.0 / 3.0;
        return ret;
    }

    private double transformLon(double x, double y) {
        double ret = 300.0 + x + 2.0 * y + 0.1 * x * x + 0.1 * x * y + 0.1 * Math.sqrt(Math.abs(x));
        ret += (20.0 * Math.sin(6.0 * x * Math.PI) + 20.0 * Math.sin(2.0 * x * Math.PI)) * 2.0 / 3.0;
        ret += (20.0 * Math.sin(x * Math.PI) + 40.0 * Math.sin(x / 3.0 * Math.PI)) * 2.0 / 3.0;
        ret += (150.0 * Math.sin(x / 12.0 * Math.PI) + 300.0 * Math.sin(x / 30.0 * Math.PI)) * 2.0 / 3.0;
        return ret;
    }
}
